/*
			 Interface Configuration

	Gets the interface's configuration values using ifconfig(8).
 */

#ifndef IF_CONFIG_H
#define IF_CONFIG_H

#include <glib.h>
#include "in_address.h"


/*
 *	Hardware (or MAC) Address:
 *
 *	For ethernet device IDs ("serial numbers").
 */
typedef struct {
	/* Indices 0 to 2 are the manufacturer's code which should be
	 * identical to all ethernet cards made by the same manufacturer,
	 * indicies 3 to 5 is the device ID ("serial number") which
	 * should not be the same for any two devices made by the same
	 * manufacturer */
	guint8		u8[6];
} IFHardwareAddress;


/*
 *	Flags:
 */
typedef enum {
	IF_CONFIG_FLAG_UP		= (1 << 0),	/* Kernel/OS drivers
							 * or modules are
							 * loaded */
	IF_CONFIG_FLAG_LOOPBACK		= (1 << 1),	/* Local (lo) device */
	IF_CONFIG_FLAG_BROADCAST	= (1 << 2),	/* Ethernet device
							 * supports broadcasting
							 * and thus can obtain
							 * IP address via DHCP */
	IF_CONFIG_FLAG_POINTTOPOINT	= (1 << 3),	/* PPP (aka POINTOPOINT) */
	IF_CONFIG_FLAG_NOTRAILERS	= (1 << 4),	/* Trailer encapsulation
							 * is disabled */
	IF_CONFIG_FLAG_RUNNING		= (1 << 5),	/* Ready to accept data */
	IF_CONFIG_FLAG_NOARP		= (1 << 6),
	IF_CONFIG_FLAG_MULTICAST	= (1 << 7)	/* Ethernet interface
							 * supports multicasting
							 * and thus can send out
							 * to multiple machines
							 * if they are tuned in */
} IFConfigFlags;


/*
 *	IFConfig Values:
 */
typedef struct {
	gchar		link_encap[80];		/* Name of interface device
						 * type */
	IFHardwareAddress	hw_addr;	/* Ethernet device ID ("serial
						 * number") */
	INAddress	inet_addr,		/* Machine internet protocol
						 * (IP) address */
			bcast,			/* Broadcast address */
			ptp,			/* Point to point address */
			mask;			/* Netmask (size of network) */
	gulong		mtu,			/* Maximum transmit unit */
			metric;			/* Packet routing priority of
						 * the device or number of
						 * "hops" to the target */
	IFConfigFlags	flags;
	gulong		rx_bytes,		/* Total number of bytes
						 * received */
			rx_packets,		/* Total number of packets
						 * received */
			rx_errors,		/* Packets received with
                                                 * invalid or insufficient
                                                 * sizes */
			rx_dropped,		/* Packets discarded due to   
                                                 * insufficient system capacity
                                                 * or memory */
			rx_overruns,		/* Packets arriving faster than
						 * the kernel can service them
						 * during the last interrupt */
			rx_frame,		/* Number of invalid packets
						 * received that are not
						 * multiples of 8 bits */
			tx_bytes,		/* Total number of bytes
						 * transmitted */
			tx_packets,		/* Total number of packets
						 * transmitted */
			tx_errors,		/* Packets encountered with
						 * invalid or insufficient
						 * sizes */
			tx_dropped,		/* Packets discarded due to
						 * insufficient system capacity
						 * or memory */
			tx_overruns,		/* Packets arriving faster than
                                                 * the kernel can service them
                                                 * during the last interrupt */
			tx_carrier,		/* Number of times the device
						 * failed to detect a carrier
						 * signal in order to transmit */
			collisions,		/* Packets colliding while
						 * traversing the network,
						 * values greater than 0
						 * suggests network congestion */
			tx_queue_len;		/* Length of the transmit
						 * queue of the device */
	gint		interrupt;		/* Interrupt request that the
						 * interface device is using
						 * on the machine */
	INAddress	base_address;
} IFConfig;


extern gboolean if_get_state(const gchar *if_name);

extern gint if_get_config(
	const gchar *if_name,
	IFConfig *config_buf
);


#endif	/* IF_CONFIG_H */
