#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>
#include <paths.h>
#include <glib.h>
#include "if_list.h"
#include "config.h"


GList *if_list(void);
void if_list_delete(GList *list);


#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)	(((s) != NULL) ? g_strdup(s) : NULL)

#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
#define STRLEN(s)	(((s) != NULL) ? (gint)strlen(s) : 0)
#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)
#define STRPFX(s,p)     ((((s) != NULL) && ((p) != NULL)) ? \
			(!strncmp((s), (p), strlen(p))) : FALSE)
#define ISBLANK(c)		(((c) == ' ') || ((c) == '\t'))

#ifndef _PATH_PROC_NET_DEV
# warning "_PATH_PROC_NET_DEV not defined, assuming /proc/net/dev"
# define _PATH_PROC_NET_DEV		"/proc/net/dev"
#endif


/*
 *	Gets the list of configured interfaces.
 *
 *	Returns a dynamically allocated GList of gchar * interface
 *	names.
 */
GList *if_list(void)
{
	gint len;
	const gchar	*s,
			*s_end;
	gchar		*if_name,
			buf[1024];
	GList *list;

	FILE *fp = fopen(_PATH_PROC_NET_DEV, "rb");
	if(fp == NULL)
		return(NULL);

	list = NULL;
	while(fgets(buf, sizeof(buf), fp) != NULL)
	{
		s = buf;
		while(ISBLANK(*s))
			s++;

		for(s_end = s; *s_end != '\0'; s_end++)
		{
			if(*s_end == ':')
				break;
		}
		if(*s_end == '\0')
			continue;

		len = s_end - s;
		if(len <= 0)
			continue;

		if_name = (gchar *)g_malloc((len + 1) * sizeof(gchar));
		if(if_name == NULL)
			break;

		(void)memcpy(
			if_name,
			s,
			(size_t)len
		);
		if_name[len] = '\0';

		list = g_list_append(
			list,
			if_name
		);
	}

	(void)fclose(fp);

	return(list);
}

/*
 *	Deletes the interface names list.
 *
 *	The list specifies a GList of gchar * interface names.
 */
void if_list_delete(GList *list)
{
	if(list == NULL)
		return;

	g_list_foreach(list, (GFunc)g_free, NULL);
	g_list_free(list);
}
