#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <paths.h>
#include <glib.h>
#include "if_stat.h"
#include "config.h"


gint if_stat(
	const gchar *if_name,
	IFStat *stat_buf
);


#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)	(((s) != NULL) ? g_strdup(s) : NULL)

#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))

#define STRLEN(s)	(((s) != NULL) ? (gint)strlen(s) : 0)
#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)
#define STRPFX(s,p)     ((((s) != NULL) && ((p) != NULL)) ? \
			(!strncmp((s), (p), strlen(p))) : FALSE)

#define ISBLANK(c)	(((c) == ' ') || ((c) == '\t'))

#ifndef _PATH_PROC_NET_DEV
# warning "_PATH_PROC_NET_DEV not defined, assuming /proc/net/dev"
# define _PATH_PROC_NET_DEV		"/proc/net/dev"
#endif


/*
 *	Gets the interface's statistics.
 */
gint if_stat(
	const gchar *if_name,
	IFStat *stat_buf
)
{
	const gchar *s;
	gchar buf[1024];
	FILE *fp;

	if(stat_buf == NULL)
	{
		errno = EINVAL;
		return(-2);
	}

	(void)memset(
		stat_buf,
		0x00,
		sizeof(IFStat)
	);

	if(if_name == NULL)
	{
		errno = EINVAL;
		return(-2);
	}

	fp = fopen(_PATH_PROC_NET_DEV, "rb");
	if(fp == NULL)
		return(-1);

	while(fgets(buf, sizeof(buf), fp) != NULL)
	{
		s = buf;
		while(ISBLANK(*s))
			s++;

		/* Header? */
		if(strchr((const char *)s, '|') != NULL)
		{
/* TODO count the number of arguments and use them to determine how
 * many arguments we should parse for the interface's line
 */
			continue;
		}

		if(!STRPFX(s, if_name))
			continue;

		s += strlen(if_name);
		while(ISBLANK(*s))
			s++;

		if(*s == ':')
			s++;
		else
			continue;

		while(ISBLANK(*s))
			s++;

#define SEEK_NEXT				\
 while((*s != '\0') && !ISBLANK(*s))		\
  s++;						\
 while(ISBLANK(*s))				\
  s++;

		stat_buf->rx_bytes = (gulong)atol(s);
		SEEK_NEXT
		stat_buf->rx_packets = (gulong)atol(s);
		SEEK_NEXT
		stat_buf->rx_errors = (gulong)atol(s);
		SEEK_NEXT
		stat_buf->rx_dropped = (gulong)atol(s);
		SEEK_NEXT
		stat_buf->rx_fifo = (gulong)atol(s);
		SEEK_NEXT
		stat_buf->rx_frame = (gulong)atol(s);
		SEEK_NEXT
		stat_buf->rx_compressed = (gulong)atol(s);
		SEEK_NEXT
		stat_buf->rx_multicast = (gulong)atol(s);
		SEEK_NEXT

		stat_buf->tx_bytes = (gulong)atol(s);
		SEEK_NEXT
		stat_buf->tx_packets = (gulong)atol(s);
		SEEK_NEXT
		stat_buf->tx_errors = (gulong)atol(s);
		SEEK_NEXT
		stat_buf->tx_dropped = (gulong)atol(s);
		SEEK_NEXT
		stat_buf->tx_fifo = (gulong)atol(s);
		SEEK_NEXT
		stat_buf->tx_colls = (gulong)atol(s);
		SEEK_NEXT
		stat_buf->tx_carrier = (gulong)atol(s);
		SEEK_NEXT
		stat_buf->tx_compressed = (gulong)atol(s);

		(void)fclose(fp);

		return(0);
	}

	(void)fclose(fp);

	errno = ENOENT;

	return(-1);
}
