#include <signal.h>
#include <gtk/gtk.h>
#include "ifm_win.h"
#include "ifm_cb.h"
#include "ifm_core.h"
#include "config.h"


void ifm_signal_cb(int s);

gint ifm_timeout_cb(gpointer data);


#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)	(((s) != NULL) ? g_strdup(s) : NULL)

#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)


/*
 *	UNIX signal callback.
 */
void ifm_signal_cb(int s)
{
	switch(s)
	{
#ifdef SIGINT
	  case SIGINT:
		exit(0);
		break;
#endif
#ifdef SIGTERM
	  case SIGTERM:
		exit(0);
		break;
#endif
#ifdef SIGQUIT
	  case SIGQUIT:
		exit(0);
		break;
#endif
#ifdef SIGSEGV
	  case SIGSEGV:
		exit(1);
		break;
#endif
	}
}


/*
 *	Main timeout callback.
 */
gint ifm_timeout_cb(gpointer data)
{
	gint nwindows_mapped;
	IFMCore *core = IFM_CORE(data);
	if(core == NULL)
		return(FALSE);

	nwindows_mapped = 0;

	if(core->ifm_wins_list != NULL)
	{
		GList *glist;
		IFMWin *win;
		for(glist = core->ifm_wins_list;
		    glist != NULL;
		    glist = g_list_next(glist)
		)
		{
			win = IFM_WIN(glist->data);
			if(win == NULL)
				continue;

			if(ifm_win_is_mapped(win))
				nwindows_mapped++;
		}
	}

	if(nwindows_mapped <= 0)
	{
		gtk_main_quit();
		return(FALSE);
	}

	return(TRUE);
}
