/*
		       Interface Monitor - Window
 */

#ifndef IFM_WIN_H
#define IFM_WIN_H

#include <gtk/gtk.h>
#include "if_config.h"
#include "if_con_info.h"
#include "if_stat.h"
#include "ifm_win_pixmap.h"
#include "ifm_win_gauge.h"
#include "ifm_win_graph.h"
#include "ifm_prop_dlg.h"


typedef struct _IFMWin			IFMWin;
#define IFM_WIN(p)			((IFMWin *)(p))


/*
 *	Flags:
 */
typedef enum {
	IFM_WIN_MAPPED			= (1 << 0),
	IFM_WIN_REALIZED		= (1 << 1),
	IFM_WIN_SHOW_BORDER		= (1 << 4),	/* Show WM border */
	IFM_WIN_SHOW_TITLE		= (1 << 5),	/* Show WM title bar */
	IFM_WIN_ANIMATE_WM_ICON		= (1 << 6),	/* Animate the WM Icon */
	IFM_WIN_SHOW_INTERFACE_STATE_TIP = (1 << 8),
	IFM_WIN_SHOW_TRAFFIC_HISTORY_TIP = (1 << 9),
	IFM_WIN_DRAG_MOVE		= (1 << 10),	/* Window is being
							 * moved by drag */
	IFM_WIN_CONNECT_PENDING		= (1 << 16)	/* This window initiated
							 * and is waiting for
							 * a connect or
							 * reconnect */
} IFMWinFlags;


/*
 *	Display Flags:
 */
typedef enum {
	IFM_WIN_DISPLAY_INTERFACE_STATE	= (1 << 0),
	IFM_WIN_DISPLAY_TRAFFIC_HISTORY	= (1 << 1),
	IFM_WIN_DISPLAY_TRAFFIC_GAUGES	= (1 << 2)
} IFMWinDisplay;


/*
 *	Interface States:
 */
typedef enum {
	IFM_WIN_INTERFACE_STATE_UNKNOWN = -1,
	IFM_WIN_INTERFACE_STATE_DOWN	= 0,
	IFM_WIN_INTERFACE_STATE_UP	= 1
} IFMWinInterfaceState;


/*
 *	State Icon Codes:
 */
typedef enum {
	IFM_WIN_STATE_ICON_DOWN,
	IFM_WIN_STATE_ICON_CONNECTING,
	IFM_WIN_STATE_ICON_UP,
	IFM_WIN_STATE_ICON_UP_RX,
	IFM_WIN_STATE_ICON_UP_TX,
	IFM_WIN_STATE_ICON_UP_RX_TX
} IFMWinStateIconCodes;
#define IFM_WIN_NSTATE_ICONS		6


/*
 *	Interface Monitor Window:
 */
struct _IFMWin {

	GtkWidget	*toplevel;
	GtkAccelGroup	*accelgrp;
	gint		busy_count,
			freeze_count;

	IFMWinFlags	flags;
	GtkOrientation	orientation;
	IFMWinDisplay	display;
	IFMWinGaugeStyle	gauge_style;
	gint		gauge_length,
			gauge_nticks;

	gint		last_button_x,
			last_button_y;

	GdkCursor	*translate_cur;

	/* List of interface state pixmaps used for setting of the
	 * WM icon when a state changes */
	IFMWinPixmap	*wm_icon_pixmaps[IFM_WIN_NSTATE_ICONS],
			*last_wm_icon_pixmap;	/* Records the pixmap used for
						 * the current interface state */

	/* Interface State */
	IFMWinPixmap	*state_icon_pixmaps[IFM_WIN_NSTATE_ICONS],
			*last_state_icon_pixmap;
	GtkWidget	*state_toplevel,
			*state_btn,		/* Menu Button */
			*state_icon_pm,		/* Icon GtkPixmap */
			*state_menu,
			*state_menu_details_mi,
			*state_menu_connect_mi,
			*state_menu_disconnect_mi,
			*state_menu_reconnect_mi,
			*state_menu_close_mi;

	/* History Graph */
	GtkWidget	*traffic_history_toplevel;
	IFMWinGraph	*traffic_history_graph;	/* For both tx and rx */

	/* Traffic Gauges */
	GtkWidget	*traffic_gauges_toplevel;
	IFMWinGauge	*tx_traffic_gauge,
			*rx_traffic_gauge;

	/* Right Click GtkMenu */
	GtkWidget	*menu,
			*details_mi,
			*connect_mi,
			*disconnect_mi,
			*reconnect_mi,
			*show_interface_state_micheck,
			*show_traffic_history_micheck,
			*show_traffic_gauges_micheck,
			*gauge_style_continuous_micheck,
			*gauge_style_leds_micheck,
			*gauge_style_gradient_leds_micheck,
			*close_mi;

	/* Interface General Information */
	gchar		*if_name;
	IFConfig	if_config;
	gulong		bandwidth_bytes_per_sec;/* If user_set_bandwidth is
						 * TRUE then this is only
						 * set in ifm_win_new(),
						 * otherwise its value is
						 * updated in
						 * ifm_win_update_state() to
						 * the value of
						 * con_info.speed_bits_per_sec / 8 */
	gboolean	user_set_bandwidth;

	/* Interface State & Connection Information */
	guint		state_update_timeout_id;

	IFMWinInterfaceState	if_state;
	IFConInfo	con_info;		/* Last connection information
						 * obtained from
						 * if_get_con_info_from_chat_report()
						 * in ifm_win_update_state()
						 * (obtained only if
						 * chat_report_path != NULL */
	gchar		*chat_report_path;

	/* Interface Statistics */
	gulong		stats_update_int_ms;
	guint		stats_update_timeout_id;
	IFStat		if_stats;		/* Last interface statistics
						 * used to calculate deltas
						 * and updated in
						 * ifm_win_update_stats() */

	GTimer		*stat_timer;		/* Used and restarted in
						 * ifm_win_update_stats() to
						 * calculate the time lapsed
						 * between the call to
						 * if_stat() */

	/* Commands */
	gchar		*connect_cmd,
			*disconnect_cmd,
			*reconnect_cmd;

	/* Properties Dialog */
	GdkRectangle	*last_prop_dlg_geometry;
	IFMPropDlg	*prop_dlg;
	IFMWinPixmap	*prop_dlg_state_icon_pixmaps[IFM_WIN_NSTATE_ICONS];

};


extern IFMWin *ifm_win_new(
	const gchar *if_name,
	const gulong bandwidth_bytes_per_sec,
	const gboolean user_set_bandwidth,
	const GtkOrientation orientation,
	const gboolean show_border,
	const gboolean show_title,
	const gboolean animate_wm_icon,
	const IFMWinDisplay display,
	const IFMWinGaugeStyle gauge_style,
	const gint bar_length,
	const gint nticks,
	const gulong stats_update_int_ms,
	const gchar *chat_report_path
);
extern void ifm_win_set_display(
	IFMWin *win,
	const IFMWinDisplay display
);
extern void ifm_win_set_gauge_style(
	IFMWin *win,
	const IFMWinGaugeStyle gauge_style
);
extern void ifm_win_set_connect_command(
	IFMWin *win,
	const gchar *cmd
);
extern void ifm_win_set_disconnect_command(
	IFMWin *win,
	const gchar *cmd
);
extern void ifm_win_set_reconnect_command(
	IFMWin *win,
	const gchar *cmd
);
extern void ifm_win_update_stats(IFMWin *win);
extern void ifm_win_update_state(IFMWin *win);
extern IFMWinStateIconCodes ifm_win_get_state_icon_code(IFMWin *win);
extern void ifm_win_set_state_icons_by_code(
	IFMWin *win,
	const IFMWinStateIconCodes state_icon_code
);
extern void ifm_win_update_display(IFMWin *win);
extern gboolean ifm_win_is_mapped(IFMWin *win);
extern void ifm_win_map(IFMWin *win);
extern void ifm_win_unmap(IFMWin *win);
extern void ifm_win_delete(IFMWin *win);


#endif	/* IFM_WIN_H */
